#ifndef _Container_H__
#define _Container_H__

#include <vector>
#include <queue>
#include <map>
#include "common.h"

using std::vector;

namespace GtpVisibilityPreprocessor {

class TransformedMeshInstance;
class ViewCell;
class HierarchyNode;
class SceneGraphNode;
class SceneGraphLeaf;
class Intersectable;
class Polygon3;
class Ray;
class Mesh;
class KdLeaf;
class Vector3;
class IntersectableGroup;

struct IndexedBoundingBox;
struct VisibilityPoint;


/** Container storing polygons used during BSP tree construction.
*/
typedef vector<Polygon3 *> PolygonContainer;

/** Container storing a bundle of rays.
*/
typedef vector<Ray *> RayContainer;

/** Container for Mesh pointers primarily for the use within the kDTree and
    BSP hierarchies 
*/
typedef vector<Intersectable *> ObjectContainer;

/** Container for ViewCell pointers primarily for the use within the kDTree and
    BSP hierarchies 
*/
typedef vector<ViewCell *> ViewCellContainer;

/** Container for HierarchyNodes pointers primarily for the use within the kDTree and
    BSP hierarchies 
*/
typedef vector<HierarchyNode *> NodeContainer;

typedef vector<SceneGraphNode *> SceneGraphNodeContainer;

typedef vector<Mesh *> MeshContainer;

/** Container storing visibility points, i.e. locations in space storing from point visibility
	information.
*/
typedef vector<VisibilityPoint *> VisibilityPointsContainer;

typedef vector<IndexedBoundingBox> IndexedBoundingBoxContainer;

/** Container for ViewCell pointers primarily for the use within the kDTree and
    BSP hierarchies 
*/
typedef vector<KdLeaf *> KdLeafContainer;

typedef std::map<int, Intersectable *> ObjectMap;

/// default vertex container for Mesh
typedef std::vector<Vector3> VertexContainer;

/// vertex index container
typedef std::vector<int> VertexIndexContainer;

/// a container for dynamic objects
typedef vector<SceneGraphLeaf *> DynamicObjectsContainer;

// a container for intersectable groups (e.g., triangle soups)
typedef vector<IntersectableGroup *> GroupContainer;

}

#endif // _Container_H__
